/**
 * Ce sketch Arduino est destin\u00e9 \u00e0 \u00eatre utilis\u00e9 avec une t\u00e9l\u00e9commande sur liaison s\u00e9rie
 * qui envoit sur la liaison s\u00e9rie 3 octets : 
 * - 0xFF
 * - vitesse gauche (entre -100 et +100)
 * - vitesse droite (entre -100 et +100)
 * 
 * En retour, l'Arduino envoie sur la liaison s\u00e9rie
 */

#include <Servo.h>
#include <MsTimer2.h>

// constantes indiquant la position (en degr\u00e9s) neutre (roue immobile)
// \u00e0 ajuster pour chacune des roues et chacun des robots
#define NEUTRE_G  76
#define NEUTRE_D  77

// nombre de vitesses autour du neutre
#define VAR  5

#include "WProgram.h"
void setup();
void loop();
void feedback();
void gauche();
void droite();
Servo servG;
Servo servD;

// vitesses (entre -100 et 100)
int vg;
int vd;

// encodeurs
long countG;
long countD;

void setup()
{
  // communication s\u00e9rie
  Serial.begin(115200);
  Serial.println("> Connection to Pobot Easy granted.");
  // connecter les servomoteurs
  servG.attach(10);
  servD.attach(9);
  // arr\u00eater le robot
  servG.write(NEUTRE_G);
  servD.write(NEUTRE_D);
  // initialiser les vitesses
  vg = 0;
  vd = 0;
  // d\u00e9clarer les interruptions externes
  attachInterrupt(0,gauche,CHANGE);
  attachInterrupt(1,droite,CHANGE);  
  // g\u00e9rer une interruption r\u00e9guli\u00e8re
   MsTimer2::set(250, feedback); // 500ms period
   MsTimer2::start();
}

void loop()
{
  // mise \u00e0 jour des vitesses
  if (Serial.available() >= 3)
  {    
    if (Serial.read() != 0xFF) 
    {
      Serial.println("Erreur de r\u00e9ception !!");
      Serial.flush();
      return;      
    }
    vg = Serial.read();
    vd = Serial.read();
    // r\u00e9tablir les valeurs sign\u00e9es
    if (vg >= 128) {
      vg = vg-255;      
    }   
    if (vd >= 128) {
      vd = vd-255;  
    }
    // commander les servomoteurs  
    servG.write(NEUTRE_G+vg*VAR/100.0);
    servD.write(NEUTRE_D-vd*VAR/100.0);
    Serial.flush();
  }
  delay(20);
}

/** Gestion de la communication en retour  **/

// Envoyer une trame d'informations sur la liaison s\u00e9rie
void feedback()
{
  Serial.print(vg,DEC);
  Serial.print(" ");
  Serial.print(vd,DEC);
  Serial.print(" ");
  Serial.print(countG,DEC);
  Serial.print(" ");
  Serial.print(countD,DEC);
  // TODO: les capteurs par exemple
  Serial.println();
}

/** Gestion de l'odom\u00e9trie **/

// Interruption de l'odom\u00e8tre gauche
void gauche()
{
  // Gestion du sens de rotation
  if (vg <0) {
    countG--;
  } 
  else {
    countG++; 
  } 
}

// Interruption de l'odom\u00e8tre droit
void droite()
{
  // Gestion du sens de rotation
  if (vd <0) {
    countD--;   
  } 
  else {
    countD++;
  }
}


int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

